% Script:     spectrum_demo_ar.m
% Copyright (x) 2019 by S. Lawrence Marple Jr.

% This script demonstrates autoregreesive spectral analysis using
% five signal sources and five AR estimation algorithms

disp(' >> CONSULT THE README FILE BEFORE FIRST USE OF THIS SCRIPT <<');
disp(' Press space bar to continue. ');  % 6 June 2019

disp(' ');
disp(' This script demonstrates AUTOREGRESSIVE spectral analysis.');
disp(' ');

%****************************  Data Source Choice  ****************************

disp(' ** SOURCE **');
disp(' Signal source choices:');
disp('     test1987             [1]');
disp('     doppler_radar        [2]');
disp('     bat_ultrasonic_pulse [3]');
disp('     doppler_heart        [4]');
disp('     doppler_helicopter   [5]');
% disp('  your_data   [6]');
source = input(' Enter signal source number: ');
if source == 1
    load('test1987.dat')
    x = test1987(:,1) + 1i*test1987(:,2);
    T = 1;                 % sampling interval of one second
    num_psd = 2048;        % number of PSD evaluation points
elseif source == 2
    load('doppler_radar.dat')
    x = doppler_radar(:,1) + 1i*doppler_radar(:,2);
    T = 1/2500;            % sampling interval 0.4 milliseconds (ms)
    num_psd = 2048;        % number of PSD evaluation points
elseif source == 3
    load('bat_ultrasonic_pulse')
    x = bat;
    T = 7.0e-6;            % sampling interval 7 microseconds (us)
    num_psd = 2048;        % number of PSD evaluation points
elseif source == 4
    load('doppler_heart')
    x = heart;
    T = 1/6250;            % sampling interval
    num_psd = 1024;        % number of PSD evaluation points
elseif source == 5
    load('doppler_helicopter')
    x = helicopter(1:4096);
    T = 1/48000;           % sampling interval
    num_psd = 2048;        % number of PSD evaluation points
% Load your signal data here
% elseif source == 6
%     load('your_file_name')
%     x = your_variable_name;
%     T = your_sampling_interval;
%     num_psd = your_number_of_PSD_evaluation_frequencies;
else
    error(' Choice does not match available signal sources.')
end
disp(' ');

%**************  Short Signal vs Long Signal Analysis Choice  ****************

disp(' ** SHORT SIGNAL vs LONG SIGNAL ANALYSIS **')
disp(' Short signal analysis produces a 1-D frequency-only spectrum.');
disp(' Long signal analysis produces a 2-D localized frequency vs localized time gram.');
stat_choice = input(' Select short signal [1] or long signal [2] analysis: ');
if stat_choice == 2
    interval = input(' Enter analysis interval duration (# samples): ');
    int_overlap = input(' Enter overlap (# samples) between analysis intervals: ');
    if (int_overlap < 0) || (int_overlap >= interval)
        error([' Overlap must be in range  0 <= overlap < ',int2str(interval)])
    end
else
    interval = length(x);
    int_overlap = 0;
end
shift = interval - int_overlap;
num_ints = fix( (length(x) - interval)/shift) + 1;
total = num_ints*shift + int_overlap;
string = [' Selected analysis choice will use ',int2str(total)];
disp([string,' signal samples, a duration of ',num2str(total*T),' seconds.']);
disp(' ');

disp(' ** AUTOREGRESSIVE SPECTRAL ESTIMATOR PARAMETERS **')
disp(' Autoregressive (AR) spectral analysis algorithm choices:') 
disp('      1  --  Yule-Walker')
disp('      2  --  Lattice (Geometric)')
disp('      3  --  Lattice (Burg)')
disp('      4  --  Covariance Linear Prediction')
disp('      5  --  Modified Covariance Linear Prediction')
method = input('Enter number of AR algorithm choice: ');
p = input('Enter autoregressive spectral order (AR filter length) to use: ');
disp(' ');

%*****************************  Spectral Analysis  ****************************

disp(' ** PERFORMING SPECTRAL ANALYSIS BASED ON SELECTIONS **')
psd = [];
b = [];
range = 1:interval;
for k=1:num_ints
    if     method == 1, [rho,a] = yule_walker(p,x(range)); % Yule_Walker
    elseif method == 2, [rho,a] = lattice('geometric',p,x(range));% Geometric
    elseif method == 3, [rho,a] = lattice('burg',p,x(range)); % Burg
    elseif method == 4, [rho,a,rho_b,a_b] = covariance_lp(p,x(range)); % Covariance
    else [rho,a] = modcovar_lp(p,x(range));  % Modified Covariance
    end
    psd = [psd; arma_psd(num_psd,T,rho,a,b)'];
    range = range + shift;
end
disp(' ');

%******************************** Plot Analysis Results ***********************

disp(' ** PLOT PARAMETERS **')
if stat_choice == 1
    plot_psd
else
    plot_TF_gram
end
%