% Script:     spectrum_demo_rls.m

% This script demonstrates time recursive autoregressive (AR) spectral analysis
% algorithms from Chapter 9 using data sets test1987, radar and bat

disp(' >> CONSULT THE README FILE BEFORE FIRST USE OF THIS SCRIPT <<');
disp(' Press space bar to continue. ');  % 6 June 2019

disp(' ');
disp(' This script demonstrates TIME-RECURSIVE AR spectral analysis.');
disp(' ');

%****************************  Data Source Choice  ****************************

disp(' ** SOURCE **');
disp(' Signal source choices:');
disp('     test1987             [1]');
disp('     doppler_radar        [2]');
disp('     bat_ultrasonic_pulse [3]');
% disp('  your_data   [4]');
source = input(' Enter signal source number: ');
if source == 1
    load('test1987.dat')
    x = test1987(:,1) + 1i*test1987(:,2);
    T = 1;                 % sampling interval of one second
    num_psd = 2048;        % number of PSD evaluation points
elseif source == 2
    load('doppler_radar.dat')
    x = doppler_radar(:,1) + 1i*doppler_radar(:,2);
    T = 1/2500;            % sampling interval 0.4 milliseconds (ms)
    num_psd = 2048;        % number of PSD evaluation points
elseif source == 3
    load('bat_ultrasonic_pulse')
    x = bat;
    T = 7.0e-6;            % sampling interval 7 microseconds (us)
    num_psd = 2048;        % number of PSD evaluation points
% Load your signal data here
% elseif source == 4
%     load('your_file_name')
%     x = your_variable_name;
%     T = your_sampling_interval;
%     num_psd = your_number_of_PSD_evaluation_frequencies;
else
    error(' Choice does not match available signal sources.')
end
disp(' ');

%**************  Short Signal vs Long Signal Analysis Choice  ****************

disp(' ** SHORT SIGNAL vs LONG SIGNAL ANALYSIS **')
disp(' Short signal analysis produces a 1-D frequency-only spectrum.');
disp(' Long signal analysis produces a 2-D localized frequency vs localized time gram.');
stat_choice = input(' Select short signal [1] or long signal [2] analysis: ');
if stat_choice == 2
    int_overlap = input(' Enter time interval (# samples) between spectral plots: ');
else
    int_overlap = length(x);
end
interval = 2*int_overlap;
disp(' ');

%********************* Spectral Estimator Parameter Choices *******************
disp(' ** TIME RECURSIVE AR SPECTRAL ESTIMATOR PARAMETERS **')
disp(' Time recursive AR spectral analysis algorithm choices:') 
disp('      1  --  Least Mean Square (LMS)')
disp('      2  --  Fast Recursive Least Squares (RLS)')
method = input(' Enter number of AR algorithm choice: ');
p = input(' Enter autoregressive spectral order (AR filter length) to use: ');
% Note: must select  0 < omega <= 1  for fast_rls algorithm to work correctly
omega = input(' Enter adaptive factor value (mu for lms  or  omega for fast_rls): ');

%*****************************  Spectral Analysis  ****************************

a = zeros(p,1);
rho=0;
rho_b=0;
gamma=0;
a_b=a;
c=a;
xx = [a; x];
psd = [];
b = [];  % dummy empty array for using function arma_psd
range = p+1:-1:1;
init = 0;
for k=1:length(x)
    if     method == 1, a = lms(omega,xx(range),a);         % LMS
    else   [init,rho,a,rho_b,a_b,gamma,c] = ...             % Fast RLS (default)
                       fast_rls(init,omega,xx(range),rho,a,rho_b,a_b,gamma,c); 
    end
    if rem(k,int_overlap) == 0
        psd = [psd; arma_psd(num_psd,T,1,a,b)'];
    end
    range = range + 1;
end
disp(' ')

%******************************** Plot Analysis Results ***********************

disp(' ** PLOT PARAMETERS **')
if stat_choice == 1
    plot_psd
else
    plot_TF_gram
end
%